@@ -1,24 +1,25 @@
+from __future__ import annotations
+
+import logging
 import pathlib
 import random
 import socket
 import time
-from typing import Iterator, Union
+from typing import Iterator
 
-import logbook
+import ping3
 
-from qBitrr.config import PING_URLS
-
-logger = logbook.Logger("Utilities")
+ping3.EXCEPTIONS = True
 
 
-def absolute_file_paths(directory: Union[pathlib.Path, str]) -> Iterator[pathlib.Path]:
+def absolute_file_paths(directory: pathlib.Path | str) -> Iterator[pathlib.Path]:
     error = True
     while error is True:
         try:
             yield from pathlib.Path(directory).glob("**/*")
             error = False
         except FileNotFoundError as e:
-            logger.warning("{e.strerror} - {e.filename}", e=e)
+            logging.warning("%s - %s", e.strerror, e.filename)
 
 
 def validate_and_return_torrent_file(file: str) -> pathlib.Path:
@@ -27,11 +28,11 @@ def validate_and_return_torrent_file(file: str) -> pathlib.Path:
         path = path.parent.absolute()
     count = 9
     while not path.exists():
-        logger.trace(
-            "Attempt {count}/10: File does not yet exists! (Possibly being moved?) | "
-            "{path} | Sleeping for 0.1s",
-            path=path,
-            count=10 - count,
+        logging.trace(
+            "Attempt %s/10: File does not yet exists! (Possibly being moved?) | "
+            "%s | Sleeping for 0.1s",
+            path,
+            10 - count,
         )
         time.sleep(0.1)
         if count == 0:
@@ -44,11 +45,11 @@ def validate_and_return_torrent_file(file: str) -> pathlib.Path:
         if path.is_file():
             path = path.parent.absolute()
         while not path.exists():
-            logger.trace(
-                "Attempt {count}/10:File does not yet exists! (Possibly being moved?) | "
-                "{path} | Sleeping for 0.1s",
-                path=path,
-                count=10 - count,
+            logging.trace(
+                "Attempt %s/10:File does not yet exists! (Possibly being moved?) | "
+                "%s | Sleeping for 0.1s",
+                path,
+                10 - count,
             )
             time.sleep(0.1)
             if count == 0:
@@ -63,27 +64,50 @@ def validate_and_return_torrent_file(file: str) -> pathlib.Path:
 
 
 def has_internet():
+    from qBitrr.config import PING_URLS
+
     url = random.choice(PING_URLS)
     if not is_connected(url):
         return False
-    logger.trace("Successfully connected to {url}", url=url)
+    logging.trace("Successfully connected to %s", url)
     return True
 
 
-def is_connected(hostname):
+def _basic_ping(hostname):
+    host = "N/A"
     try:
         # see if we can resolve the host name -- tells us if there is
         # a DNS listening
         host = socket.gethostbyname(hostname)
         # connect to the host -- tells us if the host is actually
         # reachable
-        s = socket.create_connection((host, 80), 2)
+        s = socket.create_connection((host, 80), 5)
         s.close()
         return True
-    except Exception:
+    except Exception as e:
+        logging.trace(
+            "Error when connecting to host: %s %s %s",
+            hostname,
+            host,
+            e,
+        )
         return False
 
 
+def is_connected(hostname):
+    try:
+        ping3.ping(hostname, timeout=5)
+        return True
+    except ping3.errors.PingError as e:  # All ping3 errors are subclasses of `PingError`.
+        logging.debug(
+            "Error when connecting to host: %s %s",
+            hostname,
+            e,
+        )
+    except Exception:  # Ping3 is far more robust but may requite root access, if root access is not available then run the basic mode
+        return _basic_ping(hostname)
+
+
 class ExpiringSet:
     def __init__(self, *args, **kwargs):
         max_age_seconds = kwargs.get("max_age_seconds", 0)
@@ -95,7 +119,7 @@ class ExpiringSet:
 
     def __repr__(self):
         self.__update__()
-        return "{}({})".format(self.__class__.__name__, ", ".join(self.container.keys()))
+        return f"{self.__class__.__name__}({', '.join(self.container.keys())})"
 
     def extend(self, args):
         """Add several items at once."""
